//===========================================================================
//! @file	randgen.hpp
//! @date	08 April 2004
//! @author	John Adam Croston (AC)
//
// Copyright (c) 2004, John Adam Croston.
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// 
//!
//! This is an extremely simplified random number generator class.
//!
//! The base for this code was submitted to FlipCode.com on July 23, 2000 by
//! John W. Ratcliff. That code was released into the public domain by him on
//! the same date.
//!
//! Depending on you application you may want to convert this to use an
//! algorith with better statistical properties.
//===========================================================================
#ifndef RANDGEN_HPP
#define RANDGEN_HPP
#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


//---------------------------------------------------------------------------
//! This is an extremely simplified random number generator class.
//!
//! It keeps track of it's own state and has no hidden static variables
//! like the standard libraries rand() and srand().
//!
//! It can return random integers on the range 0 to 32767 inclusive.
//! It can return random floats on the range 0.0f to 1.0f inclusive.
//---------------------------------------------------------------------------
class RandomNumberGenerator
{
public:

	RandomNumberGenerator( const int randomSeed )
		:// Initialiser list.
		m_current( randomSeed )
	{
	}

	virtual ~RandomNumberGenerator()
	{
	}

	// Return a random evenly distributed integer between 0 and
	// RAND_MAX_VALUE.
	inline const int Get()
	{
		const int val = ( ( ( m_current = m_current * 214013L + 2531011L ) >> 16 ) & 0x7fff );

		return val;
	}

	inline const int operator()()
	{
		return Get();
	}

	// Return a random evenly distributed float between 0.0 and 1.0 inclusive.
	inline const float Portion( void )
	{
		static const int	RAND_MAX_VALUE = 32767;
		static const float	RAND_INV_MAX_F = 1.0f / static_cast< float >(RAND_MAX_VALUE);

		const float val = static_cast< float >( Get() ) * RAND_INV_MAX_F;

		return val;
	}

protected:

	int m_current;
};


#endif // RANDGEN_HPP